#!/usr/bin/env node

// src/index.ts
import fs5 from "node:fs";
import path6 from "node:path";
import * as process2 from "process";
import * as p3 from "@clack/prompts";

// src/consts.ts
import path from "path";
import { fileURLToPath } from "url";
var __filename = fileURLToPath(import.meta.url);
var PKG_ROOT = path.join(path.dirname(__filename), "../");
var DEFAULT_APP_NAME = "my-app";

// src/create-app.ts
import * as p2 from "@clack/prompts";

// src/adders/auth.ts
import path2 from "node:path";

// src/helpers/svelete-add.ts
import { join } from "path";
import {
  getEnvironment,
  getFolderInfo,
  installDependencies,
  runAdder
} from "svelte-add";
import updateJsonFile from "update-json-file";

// src/package-versions.json
var package_versions_default = {
  "@auth/core": "^0.9.0",
  "@auth/sveltekit": "^0.3.5",
  "@prisma/client": "^5.0.0",
  prisma: "^5.0.0"
};

// src/helpers/svelete-add.ts
var environment = await getEnvironment();
async function runSvelteAdd({
  projectDirectory,
  adder,
  options: options2
}) {
  await runAdder({
    adder,
    projectDirectory,
    environment,
    options: options2 ?? {},
    folderInfo: await getFolderInfo({ projectDirectory })
  });
}
async function installDeps(projectDirectory, packageManager) {
  await installDependencies({
    projectDirectory,
    packageManager,
    platform: environment.platform
  });
}
async function updatePackageJson({
  projectDirectory,
  update
}) {
  await updateJsonFile(
    join(projectDirectory, "package.json"),
    (pkgJson) => {
      return update(pkgJson);
    }
  );
}
async function installPackage({
  projectDirectory,
  prod = false,
  package: pkg,
  versionOverride
}) {
  await updatePackageJson({
    projectDirectory,
    update(obj) {
      const version = versionOverride ?? package_versions_default[pkg] ?? "latest";
      if (prod) {
        if (!obj.dependencies)
          obj.dependencies = {};
        obj.dependencies[pkg] = version;
      } else {
        if (!obj.devDependencies)
          obj.devDependencies = {};
        obj.devDependencies[pkg] = version;
      }
      return obj;
    }
  });
}

// src/adders/auth.ts
import fs from "fs-extra";
async function addAuthJs(projectDirectory, {
  withDb
}) {
  await installPackage({
    projectDirectory,
    package: "@auth/core",
    prod: true
  });
  await installPackage({
    projectDirectory,
    package: "@auth/sveltekit",
    prod: true
  });
  const authTemplateDir = path2.join(PKG_ROOT, "templates/auth");
  const authSrcDir = path2.join(authTemplateDir, "src");
  const hookSrc = path2.join(
    authSrcDir,
    withDb ? "with-prisma-hooks.server.ts" : "hooks.server.ts"
  );
  const hooksDest = path2.join(projectDirectory, "src/hooks.server.ts");
  fs.copySync(hookSrc, hooksDest);
}

// src/adders/prisma.ts
import path4 from "node:path";

// src/helpers/utils.ts
import fs2 from "node:fs";
import path3 from "node:path";
function replaceTextInPath(pathToFileOrDirectory, search, replacement) {
  if (fs2.statSync(pathToFileOrDirectory).isDirectory()) {
    const files = fs2.readdirSync(pathToFileOrDirectory);
    files.forEach((file) => {
      const filePath = path3.join(pathToFileOrDirectory, file);
      replaceTextInPath(filePath, search, replacement);
    });
  } else {
    const data = fs2.readFileSync(pathToFileOrDirectory, "utf8");
    const updatedData = data.replace(new RegExp(search, "g"), replacement);
    fs2.writeFileSync(pathToFileOrDirectory, updatedData, "utf8");
  }
}
var getUserPkgManager = () => {
  const userAgent = process.env.npm_config_user_agent;
  if (userAgent) {
    if (userAgent.startsWith("yarn")) {
      return "yarn";
    } else if (userAgent.startsWith("pnpm")) {
      return "pnpm";
    } else {
      return "npm";
    }
  } else {
    return "npm";
  }
};

// src/adders/prisma.ts
import fs3 from "fs-extra";
async function addPrisma(projectDirectory, {
  withAuth,
  db: db2
}) {
  await installPackage({ projectDirectory, prod: false, package: "prisma" });
  await installPackage({
    projectDirectory,
    prod: true,
    package: "@prisma/client"
  });
  const prismaTemplateDir = path4.join(PKG_ROOT, "templates/prisma");
  const prismaDir = path4.join(prismaTemplateDir, "prisma");
  const schemaSrc = path4.join(
    prismaDir,
    withAuth ? "with-auth.prisma" : "base.prisma"
  );
  const schemaDest = path4.join(projectDirectory, "prisma/schema.prisma");
  const clientSrc = path4.join(prismaTemplateDir, "src/lib/db.ts");
  const clientDest = path4.join(projectDirectory, "src/lib/db.ts");
  await updatePackageJson({
    projectDirectory,
    update: (packageJson) => {
      packageJson.scripts = {
        ...packageJson.scripts,
        postinstall: "prisma generate"
      };
      return packageJson;
    }
  });
  fs3.copySync(schemaSrc, schemaDest);
  fs3.copySync(clientSrc, clientDest);
  if (db2 !== "sqlite") {
    replaceTextInPath(schemaDest, 'provider = "sqlite"', `provider = "${db2}"`);
  }
}

// src/helpers/git.ts
import { execSync } from "child_process";
import fs4 from "node:fs";
import path5 from "path";
import * as p from "@clack/prompts";
import { execa } from "execa";
var isGitInstalled = (dir) => {
  try {
    execSync("git --version", { cwd: dir });
    return true;
  } catch (_e) {
    return false;
  }
};
var isRootGitRepo = (dir) => {
  return fs4.existsSync(path5.join(dir, ".git"));
};
var isInsideGitRepo = async (dir) => {
  try {
    await execa("git", ["rev-parse", "--is-inside-work-tree"], {
      cwd: dir,
      stdout: "ignore"
    });
    return true;
  } catch (_e) {
    return false;
  }
};
var getGitVersion = () => {
  const stdout = execSync("git --version").toString().trim();
  const gitVersionTag = stdout.split(" ")[2];
  const major = gitVersionTag?.split(".")[0];
  const minor = gitVersionTag?.split(".")[1];
  return { major: Number(major), minor: Number(minor) };
};
var getDefaultBranch = () => {
  return execSync("git config --global init.defaultBranch || echo main").toString().trim();
};
var initializeGit = async (projectDir) => {
  const spinner3 = p.spinner();
  spinner3.start("Initializing Git...");
  if (!isGitInstalled(projectDir)) {
    spinner3.stop("Git is not installed. Skipping Git initialization.");
    return;
  }
  const isRoot = isRootGitRepo(projectDir);
  const isInside = await isInsideGitRepo(projectDir);
  const dirName = path5.parse(projectDir).name;
  if (isInside && isRoot) {
    spinner3.stop("Querying User.");
    const overwriteGit = await p.confirm({
      message: "Git is already initialized. Override?",
      initialValue: false
    });
    if (p.isCancel(overwriteGit))
      throw Error("Operation Canceled");
    if (!overwriteGit) {
      p.log.warn("Skipping git initialization.");
      return;
    }
    fs4.rmSync(path5.join(projectDir, ".git"), { recursive: true, force: true });
  } else if (isInside && !isRoot) {
    spinner3.stop("Querying User.");
    const initializeChildGitRepo = await p.confirm({
      message: `${dirName} is already in a git worktree. Would you still like to initialize a new git repository in this directory?`,
      initialValue: false
    });
    if (p.isCancel(initializeChildGitRepo))
      throw Error("Operation Canceled");
    if (!initializeChildGitRepo) {
      p.log.warn("Skipping Git initialization.");
      return;
    }
  }
  const branchName = getDefaultBranch();
  const { major, minor } = getGitVersion();
  if (major < 2 || major == 2 && minor < 28) {
    await execa("git", ["init"], { cwd: projectDir });
    await execa("git", ["symbolic-ref", "HEAD", `refs/heads/${branchName}`], {
      cwd: projectDir
    });
  } else {
    await execa("git", ["init", `--initial-branch=${branchName}`], {
      cwd: projectDir
    });
  }
  await execa("git", ["add", "."], { cwd: projectDir });
  spinner3.stop("Successfully initialized and staged git.");
};

// src/create-app.ts
import { create } from "create-svelte";
async function handleInternalCreate(projectDirectory, { tailwindcss, mdsvex, prisma, auth, installEnv, initGit }) {
  const spinner3 = p2.spinner();
  if (tailwindcss) {
    spinner3.start("Adding Tailwind CSS...");
    await runSvelteAdd({
      projectDirectory,
      adder: "tailwindcss"
    });
    spinner3.stop("Tailwind CSS added.");
  }
  if (mdsvex) {
    spinner3.start("Adding mdsvex...");
    await runSvelteAdd({
      projectDirectory,
      adder: "mdsvex"
    });
    spinner3.stop("mdsvex added.");
  }
  if (auth) {
    spinner3.start("Adding authentication...");
    await addAuthJs(projectDirectory, {
      projectDirectory,
      withDb: !!prisma
    });
    spinner3.stop("Authentication added.");
  }
  if (prisma) {
    spinner3.start("Adding Prisma...");
    await addPrisma(projectDirectory, {
      withAuth: auth,
      db: prisma
    });
    spinner3.stop("Prisma added.");
  }
  if (installEnv) {
    spinner3.start("Installing dependencies...");
    await installDeps(projectDirectory, installEnv);
    spinner3.stop("Dependencies installed.");
  }
  if (initGit) {
    await initializeGit(projectDirectory);
  }
}
async function createSvelteKitApp(projectDirectory, {
  name,
  vitest = false,
  playwright = false,
  template = "default",
  types = "typescript",
  prettier = true,
  eslint = true,
  ...internalOptions
}) {
  const spinner3 = p2.spinner();
  p2.note("Creating SvelteKit app...");
  spinner3.start("Creating SvelteKit app...");
  await create(projectDirectory, {
    name,
    types,
    prettier,
    eslint,
    vitest,
    template,
    playwright
  });
  spinner3.stop("SvelteKit app created.");
  await handleInternalCreate(projectDirectory, internalOptions);
  return Promise.resolve();
}

// src/index.ts
import { bold, cyan } from "kleur/colors";
var cwd2 = process2.argv[2] ?? ".";
p3.intro("Welcome to create-sveltek-app!");
var appName = await p3.text({
  message: "How should we call your project?",
  defaultValue: DEFAULT_APP_NAME,
  placeholder: DEFAULT_APP_NAME
});
if (p3.isCancel(appName))
  process2.exit(1);
if (appName.trim().length === 0)
  appName = DEFAULT_APP_NAME;
cwd2 = path6.join(cwd2, appName);
if (fs5.existsSync(cwd2)) {
  if (fs5.readdirSync(cwd2).length > 0) {
    const force = await p3.confirm({
      message: "Directory is not empty. Continue? (Might not work as expected!)",
      initialValue: false
    });
    if (force !== true) {
      process2.exit(1);
    }
  }
}
var features = await p3.multiselect({
  message: "Select additional options (use arrow keys/space bar)",
  required: false,
  options: [
    {
      value: "prisma",
      label: "Add Prisma for database management"
    },
    {
      value: "tailwindcss",
      label: "Add Tailwindcss for styling"
    },
    {
      value: "Auth.js",
      label: "Add Auth.js (formerly Next-Auth) for authentication"
    },
    {
      value: "mdsvex",
      label: "Add mdsvex for markdown support"
    },
    {
      value: "playwright",
      label: "Add Playwright for browser testing"
    },
    {
      value: "vitest",
      label: "Add Vitest for unit testing"
    }
  ]
});
if (p3.isCancel(features))
  process2.exit(1);
var usesPrisma = Array.isArray(features) && features.includes("prisma");
var db = null;
if (usesPrisma) {
  db = await p3.select({
    message: "Which database do you want to use?",
    initialValue: "SQLite",
    options: [
      {
        value: "sqlite",
        label: "Add SQLite"
      },
      {
        value: "postgresql",
        label: "Add PostgreSQL"
      },
      {
        value: "mongodb",
        label: "Add MongoDB"
      },
      {
        value: "mysql",
        label: "Add MySQL"
      },
      {
        value: "sqlserver",
        label: "Add SQL Server"
      },
      {
        value: "cockroachdb",
        label: "Add CockroachDB"
      }
    ]
  });
  if (p3.isCancel(db))
    process2.exit(1);
}
var options = await p3.group(
  {
    runInstallEnv: () => p3.confirm({
      message: "Do you want us to install the dependencies?",
      initialValue: true
    }).then(
      (runInstallEnv) => runInstallEnv === true ? getUserPkgManager() : false
    ),
    initGit: () => p3.confirm({
      message: "Initialize a git repository?",
      initialValue: false
    })
  },
  { onCancel: () => process2.exit(1) }
);
await createSvelteKitApp(path6.resolve(cwd2), {
  name: path6.basename(path6.resolve(cwd2)),
  playwright: features.includes("playwright"),
  vitest: features.includes("vitest"),
  tailwindcss: features.includes("tailwindcss"),
  prisma: usesPrisma ? db : false,
  mdsvex: features.includes("mdsvex"),
  auth: features.includes("Auth.js"),
  installEnv: options.runInstallEnv,
  initGit: options.initGit
}).catch((err) => {
  p3.log.error(`Something terrible happend!: ${err}`);
  process2.exit(1);
});
p3.outro("Your project is ready!");
console.log(bold("\u2714 Typescript"));
console.log('  Inside Svelte components, use <script lang="ts">\n');
console.log(bold("\u2714 ESLint"));
console.log(cyan("  https://github.com/sveltejs/eslint-plugin-svelte\n"));
console.log(bold("\u2714 Prettier"));
console.log(cyan("  https://prettier.io/docs/en/options.html"));
console.log(
  cyan("  https://github.com/sveltejs/prettier-plugin-svelte#options\n")
);
if (features.includes("playwright")) {
  console.log(bold("\u2714 Playwright"));
  console.log(cyan("  https://playwright.dev\n"));
}
if (features.includes("vitest")) {
  console.log(bold("\u2714 Vitest"));
  console.log(cyan("  https://vitest.dev\n"));
}
if (features.includes("tailwindcss")) {
  console.log(bold("\u2714 Tailwindcss"));
  console.log(cyan("  https://tailwindcss.com\n"));
}
if (usesPrisma) {
  console.log(bold("\u2714 Prisma"));
  console.log(cyan("  https://www.prisma.io\n"));
}
if (features.includes("mdsvex")) {
  console.log(bold("\u2714 mdsvex"));
  console.log(cyan("  https://mdsvex.com\n"));
}
if (features.includes("Auth.js")) {
  console.log(bold("\u2714 Auth.js"));
  console.log(cyan("  https://authjs.dev/reference/sveltekit\n"));
}
console.log("Install community-maintained integrations:");
console.log(cyan("  https://github.com/svelte-add/svelte-add"));
console.log("\nNext steps:");
var i = 1;
var relative = path6.relative(process2.cwd(), cwd2);
if (relative !== "") {
  console.log(`  ${i++}: ${bold(cyan(`cd ${relative}`))}`);
}
if (!options.runInstallEnv) {
  console.log(`  ${i++}: ${bold(cyan("npm install"))} (or pnpm install, etc)`);
}
if (!options.initGit) {
  console.log(
    `  ${i++}: ${bold(
      cyan('git init && git add -A && git commit -m "Initial commit"')
    )} (optional)`
  );
}
console.log(`  ${i++}: ${bold(cyan("npm run dev -- --open"))}`);
console.log(`
To close the dev server, hit ${bold(cyan("Ctrl-C"))}`);
console.log(`
Stuck? Visit svelte at ${cyan("https://svelte.dev/chat")}`);
//# sourceMappingURL=index.js.map